package game.model
{
   import caurina.transitions.Tweener;
   import core.BContactListener;
   import core.BonusesManager;
   import core.CoreCanvas;
   import core.CoreEngine;
   import core.events.CoreEvent;
   import flash.events.EventDispatcher;
   import flash.external.ExternalInterface;
   import game.view.AppMediator;
   import game.view.GameMediator;
   import org.puremvc.as3.patterns.proxy.Proxy;
   
   public class GameProxy extends Proxy
   {
      
      public static var contactListener:BContactListener;
      
      public static var dispatcher:EventDispatcher;
      
      public static const UPDATE:String = "Update";
      
      public static const BONUS_COLLECT:String = "BonusCollect";
      
      public static const NAME:String = "GameProxy";
      
      public static const LEVEL_NEXT:String = "LevelNext";
      
      public static const BONUS_ACTIVATE:String = "BonusActivate";
      
      public static const BONUS_DEACTIVATE:String = "BonusDeactivate";
       
      
      private var engine:CoreEngine;
      
      private var notesProxy:NotesProxy;
      
      public var isPaused:Boolean;
      
      public function GameProxy()
      {
         super(NAME);
         init();
      }
      
      private function updateHandler(e:CoreEvent) : void
      {
         var time:int = e.data as int;
         notesProxy.push(UPDATE,time);
         if(Preloader.SPONSOR_VERSION && engine.elapsedTime % 30 == 0)
         {
            ExternalInterface.call("fpBroadcastScore",getScore());
            ExternalInterface.call("fpBroadcastLevel",engine.level.num);
            ExternalInterface.call("fpSendStat","Stars",engine.bonusesManager.starsCollected,"max");
         }
      }
      
      public function getScore() : uint
      {
         return engine.score;
      }
      
      private function playerDeadHandler(e:CoreEvent) : void
      {
         endGame(0);
      }
      
      private function bonusCollectHandler(e:CoreEvent) : void
      {
         notesProxy.push(BONUS_COLLECT,e.data);
      }
      
      public function getLevelNum() : uint
      {
         return engine.level.num;
      }
      
      public function togglePause() : void
      {
         if(!isPaused)
         {
            Tweener.pauseAllTweens();
            engine.pauseSimulation();
         }
         else
         {
            Tweener.resumeAllTweens();
            engine.resumeSimulation();
         }
         isPaused = !isPaused;
      }
      
      private function levelNextHandler(e:CoreEvent) : void
      {
         notesProxy.push(LEVEL_NEXT);
      }
      
      private function gameCompleteHandler(e:CoreEvent) : void
      {
         endGame(1);
      }
      
      public function getBonuses() : Array
      {
         return engine.bonusesManager.getBonuses();
      }
      
      public function startGame() : void
      {
         var appProxy:AppProxy = facade.retrieveProxy(AppProxy.NAME) as AppProxy;
         var gameMed:GameMediator = facade.retrieveMediator(GameMediator.NAME) as GameMediator;
         var appmed:AppMediator = facade.retrieveMediator(AppMediator.NAME) as AppMediator;
         var canvas:CoreCanvas = new CoreCanvas(gameMed.getObjectsContainer(),gameMed.getCoinsContainer(),gameMed.getPlayerContainer(),gameMed.getEffectsContainer(),appmed.getPopupContainer());
         engine.gameStart(canvas,appProxy.sndTransform);
         engine.dispatcher.addEventListener(CoreEvent.UPDATE,updateHandler);
         engine.dispatcher.addEventListener(CoreEngine.PLAYER_DEAD,playerDeadHandler);
         engine.dispatcher.addEventListener(CoreEngine.LEVEL_NEXT,levelNextHandler);
         engine.dispatcher.addEventListener(CoreEvent.GAME_COMPLETE,gameCompleteHandler);
         engine.bonusesManager.addEventListener(BonusesManager.BONUS_COLLECT,bonusCollectHandler);
         engine.bonusesManager.addEventListener(BonusesManager.BONUS_ACTIVATE,bonusActivateHandler);
         engine.bonusesManager.addEventListener(BonusesManager.BONUS_DEACTIVATE,bonusDeactivateHandler);
      }
      
      private function init() : void
      {
         dispatcher = new EventDispatcher();
         notesProxy = facade.retrieveProxy(NotesProxy.NAME) as NotesProxy;
         engine = CoreEngine.getInstance();
         isPaused = false;
      }
      
      private function bonusActivateHandler(e:CoreEvent) : void
      {
         notesProxy.push(BONUS_ACTIVATE,e.data);
      }
      
      private function endGame(status:uint) : void
      {
         engine.dispatcher.removeEventListener(CoreEvent.UPDATE,updateHandler);
         engine.dispatcher.removeEventListener(CoreEngine.PLAYER_DEAD,playerDeadHandler);
         engine.dispatcher.removeEventListener(CoreEngine.LEVEL_NEXT,levelNextHandler);
         engine.dispatcher.removeEventListener(CoreEvent.GAME_COMPLETE,gameCompleteHandler);
         engine.bonusesManager.removeEventListener(BonusesManager.BONUS_COLLECT,bonusCollectHandler);
         engine.bonusesManager.removeEventListener(BonusesManager.BONUS_ACTIVATE,bonusActivateHandler);
         engine.bonusesManager.removeEventListener(BonusesManager.BONUS_DEACTIVATE,bonusDeactivateHandler);
         notesProxy.push(AppProxy.END_GAME,status);
      }
      
      private function bonusDeactivateHandler(e:CoreEvent) : void
      {
         notesProxy.push(BONUS_DEACTIVATE,e.data);
      }
   }
}
