package game.view
{
   import caurina.transitions.Tweener;
   import core.Level;
   import flash.display.Sprite;
   import flash.filters.GlowFilter;
   import flash.text.TextField;
   import flash.text.TextFieldAutoSize;
   import game.model.GameAssets;
   import game.model.GameProxy;
   import game.view.components.LevelView;
   import org.puremvc.as3.interfaces.IMediator;
   import org.puremvc.as3.interfaces.INotification;
   import org.puremvc.as3.patterns.mediator.Mediator;
   
   public class LevelMediator extends Mediator implements IMediator
   {
      
      public static const NAME:String = "LevelMediator";
       
      
      private var level:Level;
      
      private var _view:LevelView;
      
      public function LevelMediator(viewComponent:LevelView)
      {
         super(NAME,viewComponent);
         init();
      }
      
      private function createLetter(char:String) : void
      {
         var time:Number = NaN;
         var LetterClass:Class = GameAssets.getInstance().getAssetClass("LevelNumLetter");
         var letter:Sprite = new LetterClass();
         var letter_tf:TextField = letter.getChildByName("tf") as TextField;
         letter_tf.autoSize = TextFieldAutoSize.LEFT;
         letter_tf.text = char;
         letter.x = _view.lettersContainer.width + 5;
         _view.lettersContainer.addChild(letter);
         letter.y = -40 - Math.random() * 60;
         letter.alpha = 0;
         time = 0.4 + Math.random() * 0.6;
         var delay:Number = Math.random() * 0.5;
         Tweener.addTween(letter,{
            "y":0,
            "alpha":1,
            "time":time,
            "delay":delay,
            "transition":"easeoutquad"
         });
         var y:Number = 40 + Math.random() * 60;
         time = 0.4 + Math.random() * 0.6;
         Tweener.addTween(letter,{
            "y":y,
            "alpha":0,
            "time":time,
            "delay":2,
            "transition":"easeinquad",
            "onComplete":letterComplete,
            "onCompleteParams":[letter]
         });
      }
      
      override public function listNotificationInterests() : Array
      {
         return [GameProxy.LEVEL_NEXT,GameProxy.UPDATE];
      }
      
      private function update(time:int) : void
      {
         var scale:Number = (time - level.startTime) / (level.endTime - level.startTime);
         _view.progress.scaleX = scale;
      }
      
      private function levelNextAnim() : void
      {
         var char:String = null;
         var str:String = "Level " + level.num;
         for(var i:uint = 0; i < str.length; i++)
         {
            char = str.substr(i,1);
            createLetter(char);
         }
         _view.lettersContainer.x = -_view.lettersContainer.width / 2;
         var glow:GlowFilter = new GlowFilter(16777215,1,2,2,1,2);
         _view.lettersContainer.filters = [glow];
         Tweener.addTween(_view.lettersContainer,{
            "_Glow_strength":2,
            "_Glow_blurX":15,
            "_Glow_blurY":15,
            "time":0.3,
            "delay":0.7,
            "transition":"easeoutquad"
         });
         Tweener.addTween(_view.lettersContainer,{
            "_Glow_strength":1,
            "_Glow_blurX":2,
            "_Glow_blurY":2,
            "time":0.3,
            "delay":1,
            "transition":"easeoutquad"
         });
      }
      
      private function init() : void
      {
         _view = getViewComponent() as LevelView;
         _view.progress.scaleX = 0;
      }
      
      public function initialize(level:Level) : void
      {
         this.level = level;
      }
      
      override public function handleNotification(note:INotification) : void
      {
         switch(note.getName())
         {
            case GameProxy.LEVEL_NEXT:
               levelNext();
               break;
            case GameProxy.UPDATE:
               update(note.getBody() as int);
         }
      }
      
      private function levelNext() : void
      {
         _view.progress.scaleX = 0;
         _view.num_tf.text = level.num.toString();
         _view.balls_tf.text = level.balls.toString();
         levelNextAnim();
      }
      
      private function letterComplete(letter:Sprite) : void
      {
         letter.parent.removeChild(letter);
      }
   }
}
