package org.puremvc.as3.core.view
{
   import org.puremvc.as3.interfaces.*;
   import org.puremvc.as3.patterns.observer.Observer;
   
   public class View implements IView
   {
      
      protected static var instance:IView;
       
      
      protected const SINGLETON_MSG:String = "View Singleton already constructed!";
      
      protected var observerMap:Array;
      
      protected var mediatorMap:Array;
      
      public function View()
      {
         super();
         if(instance != null)
         {
            throw Error(SINGLETON_MSG);
         }
         instance = this;
         mediatorMap = new Array();
         observerMap = new Array();
         initializeView();
      }
      
      public static function getInstance() : IView
      {
         if(instance == null)
         {
            instance = new View();
         }
         return instance;
      }
      
      public function retrieveMediator(mediatorName:String) : IMediator
      {
         return mediatorMap[mediatorName];
      }
      
      public function hasMediator(mediatorName:String) : Boolean
      {
         return mediatorMap[mediatorName] != null;
      }
      
      public function notifyObservers(notification:INotification) : void
      {
         var observers:Array = null;
         var i:Number = NaN;
         var observer:IObserver = null;
         if(observerMap[notification.getName()] != null)
         {
            observers = observerMap[notification.getName()] as Array;
            for(i = 0; i < observers.length; i++)
            {
               observer = observers[i] as IObserver;
               observer.notifyObserver(notification);
            }
         }
      }
      
      protected function initializeView() : void
      {
      }
      
      public function registerMediator(mediator:IMediator) : void
      {
         var observer:Observer = null;
         var i:Number = NaN;
         mediatorMap[mediator.getMediatorName()] = mediator;
         var interests:Array = mediator.listNotificationInterests();
         if(interests.length > 0)
         {
            observer = new Observer(mediator.handleNotification,mediator);
            for(i = 0; i < interests.length; i++)
            {
               registerObserver(interests[i],observer);
            }
         }
         mediator.onRegister();
      }
      
      public function removeMediator(mediatorName:String) : IMediator
      {
         var notificationName:String = null;
         var mediator:IMediator = null;
         var observers:Array = null;
         var removalTargets:Array = null;
         var i:int = 0;
         var target:int = 0;
         for(notificationName in observerMap)
         {
            observers = observerMap[notificationName];
            removalTargets = new Array();
            for(i = 0; i < observers.length; i++)
            {
               if(Observer(observers[i]).compareNotifyContext(retrieveMediator(mediatorName)) == true)
               {
                  removalTargets.push(i);
               }
            }
            while(removalTargets.length > 0)
            {
               target = removalTargets.pop();
               observers.splice(target,1);
            }
            if(observers.length == 0)
            {
               delete observerMap[notificationName];
            }
         }
         mediator = mediatorMap[mediatorName] as IMediator;
         delete mediatorMap[mediatorName];
         if(mediator)
         {
            mediator.onRemove();
         }
         return mediator;
      }
      
      public function registerObserver(notificationName:String, observer:IObserver) : void
      {
         if(observerMap[notificationName] != null)
         {
            observerMap[notificationName].push(observer);
         }
         else
         {
            observerMap[notificationName] = [observer];
         }
      }
   }
}
